#ifndef _IO

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#define REDLED		1
#define GREENLED	0

#define REDMASK		(1<<REDLED)
#define	GREENMASK	(1<<GREENLED)

#define LEDDDR		DDRB
#define LEDPORT		PORTB

#define LEDPINSOUT() 	LEDDDR |= (REDMASK | GREENMASK)

#define REDLEDON()		LEDPORT &= ~REDMASK
#define REDLEDOFF()		LEDPORT |= REDMASK
#define GREENLEDON()	LEDPORT &= ~GREENMASK
#define GREENLEDOFF()	LEDPORT |= GREENMASK

#define ASSERTIRQ()
#define RELEASEIRQ()

#define ACTIVITYSTROBE(x)

#define NOPDelay()		{ asm("nop; nop;"); }

/* Dataport for communication with host processor */
#define DATAPORT	PORTC
#define DATAPIN		PINC
#define	DATADDR		DDRC

#define SetIOWrite()	{ DATADDR=0xFF; };
#define SetIORead() 	{ DATADDR=0x00; };

/* OE line from input latch */
#define	OEPORT	PORTA
#define	OE		4		
#define OEMASK	(1 << OE)
#define OEDDR	DDRA

#define AssertOE()	{ OEPORT &= ~OEMASK; NOPDelay(); };
#define ClearOE()	{ OEPORT |= OEMASK; };

/* LE line for output latch */
#define	LEPORT		PORTA
#define	LE			5		
#define LEMASK		(1 << LE)
#define LEDDR		DDRA

#define AssertLE()	{ LEPORT &= ~LEMASK; NOPDelay(); };
#define ClearLE()	{ LEPORT |= LEMASK; };

/* Address pin, selects either latched address or data reg */
#define ADDRPORT	PORTA
#define ADDR		0
#define ADDRMASK	(1 << ADDR)
#define ADDRDDR		DDRA

#define SelectData()	ADDRPORT &= ~ADDRMASK
#define SelectAddr()	ADDRPORT |= ADDRMASK

/* Status lines so that AVR can signal status to Dragon */
#define ST_PORT		PORTA
#define ST_DDR		DDRA
#define ST_BUSY		6
#define ST_BUSYMASK	(1<<ST_BUSY)

#define SetBusy()	ST_PORT |= ST_BUSYMASK
#define ClearBusy()	ST_PORT &= ~ST_BUSYMASK

/* When reading status from Dragon */
#define RWLine			6
#define RWMask			(1 << RWLine)
#define SAMLine			7
#define SAMMask			(1 << SAMLine)
#define SAMAddrMask		0x1E
#define SAMDataMask		0x01

#define LatchAddressIn()			{ SelectAddr(); SetIORead(); AssertOE(); LatchedAddressLast=DATAPIN; ClearOE(); }
#define ReadDataPort()				{ SelectData(); SetIORead(); AssertOE(); LatchedData=DATAPIN; ClearOE(); }
#define WriteDataPort(value)		{ SelectData(); SetIOWrite(); DATAPORT=value; AssertLE(); ClearLE(); /*log0("R:%02X\n",value);*/ }	
#define AddressPORT	

#define WASWRITE				((LatchedAddressLast & RWMask)==0) 
#define WASSAMAddr				(LatchedAddressLast & SAMMask) 

#define SAMAddr()				((LatchedAddress & SAMAddrMask) >> 1)
#define SAMData()				(LatchedAddress & SAMDataMask)

#define ReadEEPROM(addr)		eeprom_read_byte ((const uint8_t *)(addr))	
#define WriteEEPROM(addr, val)	eeprom_write_byte ((uint8_t *)(addr), (uint8_t)(val))

#define _IO
#endif
