/*
	Mega644/1284 SD/MMC bootloader for DragonMMC project.
	
	2011-09-19, P.Harvey-Smith.
*/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/boot.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <ctype.h>
#include "status.h"
#include "bootloader.h"
#include "ff.h"
#include "integer.h"
#include "../mmc/mmc2.h"
#include "../mmc/mmc2io.h"
#include "../mmc/mmc2def.h"
#include "../DragonIO.h"

uint16_t		Keys;
uint8_t 		PageData[SPM_PAGESIZE]; // create temporary location for program 
uint16_t		PageToWrite;
BYTE			configByte;
FIL				bootfile;

void RunApplication(void);
void (*JumpToApp)(void) = 0x0000;

int main(void)
{
	BYTE	result;
		
	InitIO();
 	INIT_SPI();
	sei();

	configByte=ReadEEPROM(EE_SYSFLAGS);
		
//	Serial_Init(115200,115200);
	
	// Bootload not enabled, run application.
	if(configByte&CFG_ENABLE_BOOTLOAD)
	{
		result=f_open(&bootfile,"DGNMMC.BIN",FA_OPEN_EXISTING|FA_READ);
		if(FR_OK==result)
		{
			ProgramAVR();
		}
		f_close(&bootfile);
		RunApplication();	
	}
	else
	{
		RunApplication();
	}
}

#if 0
void EraseAVR(void)
{
	uint32_t buf_address;

	buf_address=0;
	
	while(buf_address < APP_END)
	{
		boot_page_erase(buf_address);	// Perform page erase
		boot_spm_busy_wait();			// Wait until the memory is erased.
		buf_address += SPM_PAGESIZE;
	}
}
#endif 
// Erase then program a page of flash, returns address of next page in flash
uint32_t ProgramPage(uint32_t PageAddress)
{
	uint16_t	Count;
	uint16_t	ProgWord;
	uint16_t	ProgWordIdx;
	
	boot_page_erase(PageAddress);		// Erase the page
	boot_spm_busy_wait();				
	
	// Move data to tempory programming buffer
	ProgWordIdx=PageAddress;
	for(Count=0; Count<SPM_PAGESIZE; Count+=2)
	{
		ProgWord=PageData[Count];
		ProgWord|=(PageData[Count+1] << 8);
		boot_page_fill(ProgWordIdx,ProgWord);
		ProgWordIdx+=2;
	}

	boot_page_write_safe(PageAddress);	// program the page
	boot_spm_busy_wait();

	return PageAddress+SPM_PAGESIZE;	// Move to next page
}

uint8_t ProgramAVR(void)
{
//	uint8_t		XMresult;
	uint8_t		*PageDataPtr;		// Block withing PageData, that we are loading.
	uint32_t	FlashAddress;		// Page address to be programmed
	UINT 		read;				// bytes read from file
	
	PageDataPtr=PageData;			// start loading at beginning of buffer
	FlashAddress=0;					// Start programming at addr 0

	while(!f_eof(&bootfile))
	{
		if(FR_OK==f_read(&bootfile,PageDataPtr,SPM_PAGESIZE,&read))
		{
			FlashAddress=ProgramPage(FlashAddress);
		}
	}

	return PROGRAM_OK;
}
