/*
	Status.c
	
	Functions for logging program status to the serial port, to
	be used for debugging pruposes etc.
	
	2008-03-21, P.Harvey-Smith.
	
	2011-10-07, Minimilised for bootloader.
	
*/

#include <avr/interrupt.h>
#include <stdio.h>
#include <ctype.h>
#include "terminalcodes.h"
#include "status.h"

#ifdef SERIAL_STATUS

void StdioSerial_TxByte(char DataByte, uint8_t	Port)
{
	#ifdef COOKED_SERIAL	
		if((DataByte=='\r') || (DataByte=='\n'))
		{
			Serial_TxByte0('\r');
			Serial_TxByte0('\n');
		}
		else
	#endif
		Serial_TxByte0(DataByte);
	
}

int StdioSerial_TxByte0(char DataByte, FILE *Stream)
{
	StdioSerial_TxByte(DataByte,0);
	return 0;
}

void cls(uint8_t	Port)
{
	log0(ESC_ERASE_DISPLAY);
	log0(ESC_CURSOR_POS(0,0));
}

void USART_Init0(const uint32_t BaudRate)
{
#ifdef UCSR0A
	UCSR0A = 0;
	UCSR0B = ((1 << RXEN0) | (1 << TXEN0));
	UCSR0C = ((1 << UCSZ01) | (1 << UCSZ00));
	
	UBRR0  = SERIAL_UBBRVAL(BaudRate);
#else
	UCSRA = 0;
	UCSRB = ((1 << RXEN)  | (1 << TXEN));
	UCSRC = ((1 << UCSZ1) | (1 << UCSZ0));
	
	UBRR  	= SERIAL_UBBRVAL(BaudRate);
#endif
}

/** Transmits a given byte through the USART.
 *
 *  \param DataByte  Byte to transmit through the USART
 */
void Serial_TxByte0(const char DataByte)
{
#ifdef UCSR0A
	while ( !( UCSR0A & (1<<UDRE0)) )		;
	UDR0=DataByte;
#else
	while ( !( UCSRA & (1<<UDRE)) )		;
	UDR=DataByte;
#endif
}

void Serial_Init(const uint32_t BaudRate0,
				 const uint32_t BaudRate1)
{
	if (BaudRate0<=0)
		USART_Init0(DefaultBaudRate);
	else
		USART_Init0(BaudRate0);
		
	cls(0);
	
	log0("stdio initialised\n");
	log0("compiled at %s on %s\n",__TIME__,__DATE__);
	
	log0("SerialPort0\n");
}

#else

void USART_Init0(const uint32_t BaudRate) {};
void Serial_TxByte0(const char DataByte) {};

void Serial_Init(const uint32_t BaudRate0,
				 const uint32_t BaudRate1)
{};

void cls(uint8_t	Port) {};

#endif