/*
	DragonMMC	: MMC interface for the Dragon.

	2011-06-16, P.Harvey-Smith, MMC work based on AtoMMC by Charlie Robson.
	
	2011-06-28
		Prototype board completed.
	
	2011-07-02
		MMC Card directory command now working !
	
	2011-07-03
		Cassette emulation now working for cload[m] / csave[m]
		Delete command implemented.
		
	2011-09-xx
		Added ability to load and run cartridge images in the $C000-$FEFF 
		area by loading them into the DragonMMC ram area.
		
	2011-11-12
		Added directory and wildcard support to CAT and FINDFIRST/FINDNEXT
		
	2011-12-06
		Fixed initialisation of the JTAG CPLD programming lines.
		JTAG lines are now left as inputs with pullups enabled unless 
		the CPLD is being programmed. This prevents noise on the lines 
		causing the CPLD to spuriously enter programming mode.
		
	2012-10-20
		Modified SDDOS read and write sector routines to use files rather
		than base sector + displacement, which assumes that the image file
		is contiguous.
*/

#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include "status.h"
#include "dragonio.h"
#include "ff.h"
#include "dragon_disk.h"
#include "mmc/integer.h"
#include "mmc/mmc2.h"
#include "mmc/mmc2io.h"
#include "mmc/mmcio.h"
#include "mmc/diskio.h"
#include "dragon_cas.h"

// Global data for MMC

unsigned char 	globalData[GLOBUFFSIZE];
//char 			windowData[512];
BYTE 			configByte;
BYTE 			blVersion;
WORD			SAMBits;
WORD			SaveSAMBits;
BYTE			CardInserted;
BYTE			CardInsertedNow;


void InitVars(void)
{
	SAMBits=0;
	SaveSAMBits=0;
	_delay_ms(1000);
}

// Toggle the leds so that the user knows the system is ready.
void flag_init(void)
{
	GREENLEDON();
	_delay_ms(100);
	REDLEDON();
	_delay_ms(100);
	
	GREENLEDOFF();
	_delay_ms(100);
	REDLEDOFF();
}

#define NOW "compiled at " __TIME__ " on " __DATE__ "\n"

int main(void)
{
	InitVars();
	Serial_Init(115200,115200);
	log0("Dragon MMC Interface V1.0\n");
	log0("2013-03-30 Ramoth Software.\n");
	log0(NOW);
	
	log0("I/O Init\n");
	InitIO();
	configByte = ReadEEPROM(EE_SYSFLAGS);
	log0("configByte=%02X\n",configByte);
	
	CardInserted=CARD_INSERTED();
	log0("Card inserted=%02X\n",CardInserted);
	
	log0("MMC Init\n");
	INIT_SPI();
	at_initprocessor();
	
	log0("init done!\n");
	sei();

	flag_init();

	if(CPLDUpdateFlag())
	{
		REDLEDON();
		UpdateCPLD();
		REDLEDOFF();
	}

	while(1)
	{
		// Check for incomming activity from Dragon
		if(LatchInt())
		{
			REDLEDON();
			ClearLatchInt();
			at_process();
			REDLEDOFF();
		}
		
		// Check for snapshot button pressed
		if(SnapPressed())
		{
			_delay_ms(2);
			if(SnapPressed())
			{
				log0("Snapshot pressed!\n");
				WaitSnapsRelease();
				log0("Snapshot released!\n");
			}
		}
		
		// Check for new card insertion / removal
		if(CARD_INSERTED() != CardInserted)
		{
			_delay_ms(500);
			CardInsertedNow=CARD_INSERTED();
			if(CardInsertedNow)
			{
				log0("Card inserted!\n");
				if (!disk_initialize(0))
					InitNewCard();
			}
			else
			{	
				DismountCard();
				log0("Card removed!\n");
			}
			
			CardInserted=CardInsertedNow;
		}
	}
	
	return 0;
}
