#ifndef _IO

#if (PLATFORM==PLATFORM_PIC)

#define LEDPINSOUT() TRISCbits.TRISC0 = 0; TRISCbits.TRISC1 = 0;

#define REDLEDON() PORTCbits.RC0 = 1;
#define REDLEDOFF() PORTCbits.RC0 = 0;
#define GREENLEDON() PORTCbits.RC1 = 1;
#define GREENLEDOFF() PORTCbits.RC1 = 0;

#define ASSERTIRQ()  PORTCbits.RC6 = 0; TRISCbits.TRISC6 = 0;
#define RELEASEIRQ() TRISCbits.TRISC6 = 1;

#define ACTIVITYSTROBE(x) LATAbits.LATA5 = x;

#define WASWRITE TRISEbits.IBF

#define LatchAddressIn()		{ LatchedAddressLast=PORTA; }
#define ReadDataPort()
#define WriteDataPort(value)	{ LATD=value; }	

extern void redSignal(unsigned char);

#elif (PLATFORM==PLATFORM_AVR)
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#define REDLED		1
#define GREENLED	0

#define REDMASK		(1<<REDLED)
#define	GREENMASK	(1<<GREENLED)

#define LEDDDR		DDRB
#define LEDPORT		PORTB

#define LEDPINSOUT() 	LEDDDR |= (REDMASK | GREENMASK)

#define REDLEDON()		LEDPORT &= ~REDMASK
#define REDLEDOFF()		LEDPORT |= REDMASK
#define GREENLEDON()	LEDPORT &= ~GREENMASK
#define GREENLEDOFF()	LEDPORT |= GREENMASK

#define ASSERTIRQ()
#define RELEASEIRQ()

#define ACTIVITYSTROBE(x)

#define NOPDelay()		{ asm("nop; nop;"); }

/* Dataport for communication with host processor */
#define DATAPORT	PORTC
#define DATAPIN		PINC
#define	DATADDR		DDRC

#define SetIOWrite()	{ DATADDR=0xFF; };
#define SetIORead() 	{ DATADDR=0x00; };

/* OE line from input latch */
#define	OEPORT	PORTA
#define	OE		4		
#define OEMASK	(1 << OE)
#define OEDDR	DDRA

#define AssertOE()	{ OEPORT &= ~OEMASK; NOPDelay(); };
#define ClearOE()	{ OEPORT |= OEMASK; };

/* LE line for output latch */
#define	LEPORT		PORTA
#define	LE			5		
#define LEMASK		(1 << LE)
#define LEDDR		DDRA

#define AssertLE()	{ LEPORT &= ~LEMASK; NOPDelay(); };
#define ClearLE()	{ LEPORT |= LEMASK; };

/* Address pin, selects either latched address or data reg */
#define ADDRPORT	PORTA
#define ADDR		0
#define ADDRMASK	(1 << ADDR)
#define ADDRDDR		DDRA

#define SelectData()	ADDRPORT &= ~ADDRMASK
#define SelectAddr()	ADDRPORT |= ADDRMASK

/* Status lines so that AVR can signal status to Dragon */
#define ST_PORT		PORTA
#define ST_DDR		DDRA
#define ST_BUSY		6
#define ST_BUSYMASK	(1<<ST_BUSY)

#define SetBusy()	ST_PORT |= ST_BUSYMASK
#define ClearBusy()	ST_PORT &= ~ST_BUSYMASK

/* When reading status from Dragon */
#define RWLine			4
#define RWMask			(1 << RWLine)

#define LatchAddressIn()			{ SelectAddr(); SetIORead(); AssertOE(); LatchedAddressLast=DATAPIN; ClearOE(); }
#define ReadDataPort()				{ SelectData(); SetIORead(); AssertOE(); LatchedData=DATAPIN; ClearOE(); }
#define WriteDataPort(value)		{ SelectData(); SetIOWrite(); DATAPORT=value; AssertLE(); ClearLE(); /*log0("R:%02X\n",value);*/ }	
#define AddressPORT	

#define WASWRITE				((LatchedAddressLast & RWMask)==0) 

#define ReadEEPROM(addr)		eeprom_read_byte ((const uint8_t *)(addr))	
#define WriteEEPROM(addr, val)	eeprom_write_byte ((uint8_t *)(addr), (uint8_t)(val))

#elif (PLATFORM==PLATFORM_EMU)

// it's for an emulator!

#define LEDPINSOUT()

#define REDLEDON()
#define REDLEDOFF()
#define GREENLEDON()
#define GREENLEDOFF()

#define ASSERTIRQ()
#define RELEASEIRQ()

#define ACTIVITYSTROBE(x)

#define STKPTR 0

#define redSignal(x)

#endif

#define _IO
#endif
