#include "ff.h"

#include "integer.h"
#include "dragon_disk.h"
#include "mmc2.h"

#include <string.h>
#include "mmc2io.h"
#include "mmc2def.h"
#include "diskio.h"
#include "status.h"
#include "dragon_cas.h"
#include "wildcard.h"
#include <util/delay.h>

BYTE res;

BYTE globalIndex;
WORD globalAmount;
BYTE globalDataPresent;
BYTE rwmode;

DIR dir;
FIL fil;
FILINFO filinfo;
FATFS fatfs;

extern BYTE windowData[];

#define WILD_LEN	16

char	WildPattern[WILD_LEN+1];

#ifdef INCLUDE_SDDOS

imgInfo	driveInfo[SDDOS_NO_DRIVES];
FIL		driveFILE[SDDOS_NO_DRIVES];

BYTE globalCurDrive;
DWORD globalLBAOffset;

#endif



// use only immediately after open
extern void get_fileinfo_special(FILINFO *);


void at_initprocessor(void)
{
	rwmode = 0;

// fatfs.win = windowData;

#ifdef INCLUDE_SDDOS
	memset(&driveInfo[0], 0xff, sizeof(imgInfo)*SDDOS_NO_DRIVES);
	memset(&driveFILE[0], 0xff, sizeof(FIL)*SDDOS_NO_DRIVES);
#endif

	InitNewCard();
}

void InitNewCard(void)
{
  f_chdrive(0);
  log0("f_mount()=%X\n",f_mount(0, &fatfs));
}


void DismountCard(void)
{
	f_mount(0,NULL);
}

void GetWildcard(void)
{
	int	Idx			= 0;
	int	WildPos		= -1;
	int	LastSlash	= -1;
	
	//log0("GetWildcard() %s\n",(const char *)globalData);
	
	while ((Idx<strlen((const char*)globalData)) && (WildPos<0)) 
	{
		// Check for wildcard character
		if((globalData[Idx]=='?') || (globalData[Idx]=='*')) 
			WildPos=Idx;

		// Check for path seperator
		if((globalData[Idx]=='\\') || (globalData[Idx]=='/'))
			LastSlash=Idx;
			
		Idx++;
	}
	
	//log0("GetWildcard() Idx=%d, WildPos=%d, LastSlash=%d\n",Idx,WildPos,LastSlash);
	
	if(WildPos>-1)
	{
		if(LastSlash>-1)
		{
			// Path followed by wildcard
			// Terminate dir filename at last slash and copy wildcard
			globalData[LastSlash]=0x00;
			strncpy(WildPattern,(const char*)&globalData[LastSlash+1],WILD_LEN);
		}
		else
		{
			// Wildcard on it's own
			// Copy wildcard, then set path to null
			strncpy(WildPattern,(const char*)globalData,WILD_LEN);
			globalData[0]=0x00;
		}
	}
	else
	{
		// No wildcard, show all files
		strncpy_P(WildPattern,PSTR("*"),WILD_LEN);
	}
	
	//log0("GetWildcard() globalData=%s WildPattern=%s\n",(const char*)globalData,WildPattern); 
}

void wfnDirectoryOpen(void)
{
	// Separate wildcard and path 
	GetWildcard();
   
	res = f_opendir(&dir, (const char*)globalData);
	//log0("f_opendir()=%02X\n",res);
	if (FR_OK != res)
	{
		WriteStatus(STATUS_ERROR | res);
		ClearBusy();
		return;
	}

	WriteStatus(STATUS_COMPLETE);
	ClearBusy();
}


void wfnDirectoryRead(void)
{
	char len;
	int	 Match;

	while (1)
	{
		char n = 0;

		res = f_readdir(&dir, &filinfo);
		if (res != FR_OK || !filinfo.fname[0])
		{
			// done
			WriteStatusCond((res != FR_OK),(STATUS_ERROR | res),STATUS_COMPLETE);
			ClearBusy();
			return;
		}

		// Check to see if filename matches current wildcard
		//
		Match=wildcmp(WildPattern,filinfo.fname);
		//log0("WildPattern=%s, filinfo.fname=%s, Match=%d\n",WildPattern,filinfo.fname,Match);
		if(Match)
		{
			len = (char)strlen(filinfo.fname);

			if (filinfo.fattrib & AM_DIR)	
			{
				n = 1;
				globalData[0] = '<';
			}

			strcpy((char*)&globalData[n], (const char*)filinfo.fname);

			if (filinfo.fattrib & AM_DIR)
			{
				globalData[len+1] = '>';
				globalData[len+2] = 0;
				len += 2; // brackets
			}

			// just for giggles put the attribute & filesize in the buffer
			//
			globalData[len+1] = filinfo.fattrib;
			memcpy(&globalData[len+2], (void*)(&filinfo.fsize), sizeof(DWORD));

			WriteStatus(STATUS_OK);
			ClearBusy();
			return;
		}
	}
}



void wfnSetCWDirectory(void)
{
	res = f_chdir((const TCHAR*)globalData);
	
	WriteStatusCond((res != FR_OK),(STATUS_ERROR | res),STATUS_COMPLETE);  
	ClearBusy();
}

void wfnGetCWDirectory(void)
{
	// Get current working directory and return it as a count + string
	res = f_getcwd((TCHAR*)globalData,GLOBUFFSIZE-2);

    WriteStatusCond((res != FR_OK),(STATUS_ERROR | res),STATUS_COMPLETE);
    ClearBusy();
}

BYTE fileOpen(BYTE mode)
{
	BYTE result;
	log0("Opening file : %s, mode : %02X\n",(const char*)globalData, mode);
	result = f_open(&fil, (const char*)globalData, mode);
	log0("result:%02X\n",result);

	return result;
}

void wfnFileOpenRead(void)
{
	res = fileOpen(FA_OPEN_EXISTING|FA_READ);
//   get_fileinfo_special(&filinfo);
	WriteStatusCond((res != FR_OK),(STATUS_ERROR | res),STATUS_COMPLETE);	
	ClearBusy();
}

void wfnFileOpenWrite(void)
{
	res = fileOpen(FA_CREATE_NEW|FA_WRITE);
	
	WriteStatusCond((res != FR_OK),(STATUS_ERROR | res),STATUS_COMPLETE);		
	ClearBusy();
}


void wfnFileGetInfo(void)
{
   union
   {
      DWORD dword;
      char byte[4];
   }
   dwb;

   dwb.dword = fil.fsize;
   globalData[0] = dwb.byte[0];
   globalData[1] = dwb.byte[1];
   globalData[2] = dwb.byte[2];
   globalData[3] = dwb.byte[3];

   dwb.dword = (DWORD)(fil.sclust-2) * fatfs.csize + fatfs.database;
   globalData[4] = dwb.byte[0];
   globalData[5] = dwb.byte[1];
   globalData[6] = dwb.byte[2];
   globalData[7] = dwb.byte[3];

   dwb.dword = fil.fptr;
   globalData[8] = dwb.byte[0];
   globalData[9] = dwb.byte[1];
   globalData[10] = dwb.byte[2];
   globalData[11] = dwb.byte[3];

   globalData[12] = filinfo.fattrib & 0x3f;

   WriteStatus(STATUS_COMPLETE);
   ClearBusy();
}

// Read 1 byte and return to caller.
unsigned char fileRead(void)
{
	UINT read;
	
	f_read(&fil, globalData, 1, &read);
	
	return globalData[0];
}

void wfnFileRead(void)
{
	UINT read;
	if (globalAmount == 0)
	{
		globalAmount = 256;
	}

	res = f_read(&fil, globalData, globalAmount, &read);
	WriteStatusCond((res != FR_OK),(STATUS_ERROR | res),STATUS_OK);		
	
	//log0("Read %d bytes of %d requested\n",read,globalAmount);
	ClearBusy();
}




void wfnFileWrite(void)
{
	UINT written;
	if (globalAmount == 0)
	{
		globalAmount = 256;
	}

	res = f_write(&fil, (void*)globalData, globalAmount, &written);
//log0("wfnFileWrite() %d bytes written\n",written);
	WriteStatusCond((res != FR_OK),(STATUS_ERROR | res),STATUS_COMPLETE);	
	ClearBusy();
}


BYTE fileClose(void)
{
	FRESULT	res_close;
	FRESULT	res_sync;
		
	res_sync=f_sync(&fil);
	res_close=f_close(&fil);

//log0("f_sync():%02X, f_close():%02X\n",res_sync,res_close);
	return res_close;
}

void wfnFileClose(void)
{
	res = fileClose();
	
	WriteStatusCond((res != FR_OK),(STATUS_ERROR | res),STATUS_COMPLETE);	
	ClearBusy();
}

void wfnFileDelete(void)
{
	res = f_unlink((const TCHAR*)&globalData[0]);
	
	WriteStatusCond((res != FR_OK),(STATUS_ERROR | res),STATUS_COMPLETE);	
	ClearBusy();
}


#ifdef INCLUDE_SDDOS


BYTE tryOpenImage(BYTE 	ImageNo)
{
	BYTE 		DriveNo;
	VDK_HEADER	Head;
	UINT		read;
	
	// Note file will be created if it does not exist, this allows us to create images
	
	res = f_open(&driveFILE[ImageNo], (const TCHAR*)&driveInfo[ImageNo].filename,FA_READ | FA_WRITE | FA_OPEN_ALWAYS);

	log0("tryOpenImage(): res=%02X\n",res);

	if (FR_OK != res) 
	{
		return STATUS_ERROR | res;
	}

	for (DriveNo = 0; DriveNo < SDDOS_NO_DRIVES; ++DriveNo)
	{
		if (ImageNo == DriveNo)
		{
			continue;
		}

		if (memcmp((void*)&driveInfo[ImageNo], (void*)&driveInfo[DriveNo], sizeof(imgInfo)) == 0)
		{
			// warning - already mounted
			return STATUS_ERROR + ERROR_ALREADY_MOUNT; // 0x4a;
		}
	}

	// assume no header
	driveInfo[ImageNo].HeadSize=0;
	
	if(FR_OK == f_read(&driveFILE[ImageNo],&Head,sizeof(Head),&read))
	{
		if(VALID_VDK(&Head))
		{
			SET_GEOM(&driveInfo[ImageNo].geom,Head.NoTracks,(Head.NoSides*DDSS_SECTORS));
			driveInfo[ImageNo].HeadSize=Head.Headsize;
		}
	}

	// all good - should only call "get_fileinfo_special" if no other file operations
	// have occurred since the last open.
	//
//   get_fileinfo_special(&filinfo);
//	driveInfo[ImageNo].attribs = filinfo.fattrib;

//	return driveInfo[ImageNo].attribs;
	return STATUS_COMPLETE;
}



void saveDrivesImpl(void)
{
	log0("Writing BOOTDRV.CFG\n");
	strcpy_P((char*)&globalData[0],PSTR("BOOTDRV.CFG"));

	res = f_open(&fil, (const TCHAR*)globalData, FA_OPEN_ALWAYS|FA_WRITE);
	if (FR_OK == res)
	{
		UINT temp;
		f_write(&fil, (const void*)&driveInfo[0], SDDOS_NO_DRIVES * sizeof(imgInfo), &temp);
		f_close(&fil);
		log0("Wrote %d bytes\n",temp);
	}
}


void wfnOpenSDDOSImg(void)
{
	// globalData[0] = drive number 0..3
	// globalData[1]... image filename
	// if byteValueLatch is true, file will be mounted even if it doesnt exist. 

	BYTE stat, error;
	BYTE id = globalData[0] & 3;

	stat = driveInfo[id].attribs;

	memset(&driveInfo[id], 0, sizeof(imgInfo));
	strncpy((char*)&driveInfo[id].filename, (const char*)&globalData[1], 13);

	log0("wfnOpenSDDOSImg():%s\n",driveInfo[id].filename);

	error = tryOpenImage(id);
	if (error & STATUS_ERROR)
	{
		// fatal error range
		//
		memset(&driveInfo[id], 0xff, sizeof(imgInfo));
	}

	// always save - even if there was an error
	// we may have nullified a previously valid slot.
	//
	saveDrivesImpl();

	WriteStatus(error);
	ClearBusy();
}

BYTE SDDOS_seek(BYTE	drive,		// Drive ID
			    DWORD	LBA,		// LBA of sector to access
				BYTE	offset)		// Offset within sector, usually 0
{
	DWORD 	fpos=(LBA * DDOS_SECTOR_SIZE)+offset+driveInfo[drive].HeadSize;

	log0("SDDOS_seek() %d [%08X]\n",fpos,fpos);
		
	return f_lseek(&driveFILE[drive],fpos);
}

void wfnReadSDDOSSect(void)
{
	BYTE 	returnCode = STATUS_ERROR | ERROR_INVALID_DRIVE;
	UINT	bytes_read;

	log0("wfnReadSDDOSSect() Drive=%d, LBA=%d, offset=%d\n",globalCurDrive,globalLBAOffset,driveInfo[globalCurDrive].HeadSize);
	log0("attrib=%02X\n",driveInfo[globalCurDrive].attribs);
	
	if (driveInfo[globalCurDrive].attribs != 0xff)
	{
		if(FR_OK==SDDOS_seek(globalCurDrive,globalLBAOffset,driveInfo[globalCurDrive].HeadSize))
		{
			returnCode = f_read(&driveFILE[globalCurDrive], globalData, SDDOS_SECTOR_SIZE, &bytes_read);
		}

log0("f_read(): result=%02x\n",returnCode);

		if (RES_OK == returnCode)
		{
			HexDumpHead(&globalData[0],SDDOS_SECTOR_SIZE,0);
			
			WriteStatus(STATUS_COMPLETE);
			ClearBusy();
			return;
		}

		driveInfo[globalCurDrive].attribs = 0xff;
		returnCode |= STATUS_ERROR;
	}

	WriteStatus(returnCode);
	ClearBusy();
}




void wfnWriteSDDOSSect(void)
{
	BYTE returnCode = STATUS_ERROR | ERROR_INVALID_DRIVE;
	UINT bytes_written;

	if (driveInfo[globalCurDrive].attribs != 0xff)
	{
		if (driveInfo[globalCurDrive].attribs & 1)
		{
			// read-only
			WriteStatus(STATUS_ERROR | ERROR_READ_ONLY);
			return;
		}

		if(FR_OK==SDDOS_seek(globalCurDrive,globalLBAOffset,0))
		{
			returnCode = f_write(&driveFILE[globalCurDrive], globalData, SDDOS_SECTOR_SIZE, &bytes_written);
		}

		// invalidate the drive on error
		if(FR_OK==returnCode)
		{
			WriteStatus(STATUS_COMPLETE);
			ClearBusy();
			return;
		}

		driveInfo[globalCurDrive].attribs = 0xff;
		returnCode |= STATUS_ERROR;
	}

	WriteStatus(returnCode);
	ClearBusy();
}


void wfnValidateSDDOSDrives(void)
{
	BYTE DriveID;
	BYTE* ii = (BYTE*)driveInfo;

	// read the imgInfo structures back out of eeprom,
	// or 'BOOTDRV.CFG' if present (gets precidence)
	strcpy_P((char*)&globalData[0],PSTR("BOOTDRV.CFG"));


	// try to read the boot config file
	//
	res = f_open(&fil, (const char*)globalData, FA_READ|FA_OPEN_EXISTING);
	if (!res)
	{
		UINT temp;
		res = f_read(&fil, (void*)(&ii[0]), SDDOS_NO_DRIVES * sizeof(imgInfo), &temp);
	}
	else
	{
		memset(&ii[0], 0xff, SDDOS_NO_DRIVES * sizeof(imgInfo));
	}

	for (DriveID = 0; DriveID < SDDOS_NO_DRIVES; ++DriveID)
	{
		if (driveInfo[DriveID].attribs == 0xff || (tryOpenImage(DriveID) & 0x40))
		{
			memset(&driveInfo[DriveID], 0xff, sizeof(imgInfo));
		}
	}

	saveDrivesImpl();

	WriteStatus(STATUS_COMPLETE);
	ClearBusy();
}


void wfnSerialiseSDDOSDrives(void)
{
   saveDrivesImpl();
   WriteStatus(STATUS_COMPLETE);
   ClearBusy();
}


// slightly uneasy about using this var, but it should be fine.
//
extern BYTE byteValueLatch;

void wfnUnmountSDDOSImg(void)
{
   BYTE	DriveID = byteValueLatch & 3;
   f_close(&driveFILE[DriveID]);
   
   memset(&driveInfo[DriveID], 0xff, sizeof(imgInfo));

   saveDrivesImpl();
   WriteStatus(STATUS_COMPLETE);
   ClearBusy();
}


void wfnGetSDDOSImgNames(void)
{
   BYTE i;
   BYTE m, n = 0;
   for (i = 0; i < SDDOS_NO_DRIVES; ++i)
   {
      if (driveInfo[i].attribs != 0xff)
      {
         m = 0;

         while(driveInfo[i].filename[m] && m < 12)
         {
            globalData[n] = driveInfo[i].filename[m];
            ++m;
            ++n;
         }
      }
      globalData[n] = 0;
      ++n;
   }

   WriteStatus(STATUS_COMPLETE);
   ClearBusy();
}

BYTE ReadSetGeom(BYTE		DriveID,
				 DWORD		LBA,
				 DISK_GEOM	*geom)
{
	UINT		read;
    BYTE		error;
 
	error=SDDOS_seek(DriveID,LBA,DD_GEOM_OFFSET);
	if(FR_OK==error)
	{
		error=f_read(&driveFILE[DriveID],geom,sizeof(DISK_GEOM),&read);
		
		if(read!=sizeof(DISK_GEOM))
		{
			error=FR_DISK_ERR;
			memset(geom,0xFF,sizeof(DISK_GEOM));	
		}
		
		// Invalid geometry, flag it
		if(!GEOM_VALID(geom))
			error=FR_DISK_ERR;
	}
	
	return error;
}

// get the geometry of an image, tries to guess the geometry of
// a DragonDOS image by looking for the directory track at either 
// 18x20 sectors in for single sided or 18x20x2 sectors in for 
// double sided
void wfnGetImgGeometry(void)
{
	BYTE		DriveID	= byteValueLatch;
	DISK_GEOM	*geom 	= &driveInfo[DriveID].geom;
	BYTE		error = STATUS_ERROR | ERROR_INVALID_DRIVE;
		
	// If drive info is valid, then return it
	if(!GEOM_VALID(geom))
	{
		// Try reading both ds dir tracks 
		error = ReadSetGeom(DriveID,LBA_DIR_DS,geom);
		
		if(FR_OK != error)
			error = ReadSetGeom(DriveID,LBA_DIR_SS,geom);


		if(FR_OK == error)
			error |= STATUS_COMPLETE;
	}
	
	log0("wfnGetImgGeometry() Drive=%d, Sec/Track=%d, Tracks=%d\n",DriveID,geom->SecPerTrack,geom->Tracks);
	
	globalData[0]=geom->SecPerTrack;
	globalData[1]=geom->Tracks;
	
	HexDumpHead(&globalData[0],16,0);
	
	WriteStatus(error);
	ClearBusy();
}

void wfnCreateImg(void)
{
	SDDOS_seek(globalCurDrive,globalLBAOffset,0);
	WriteStatus(STATUS_COMPLETE);
	ClearBusy();
}

#endif

void wfnGetCasFiletype(void)
{
	DWORD				old_fp;
	CASFilenameBlock	FNBlock;
	BYTE				last_byte;
	UINT 				read;
	
	// Assume that the test will fail !
	globalData[0]=0xFF;
	 
	if(!(fil.fs==0))
	{
		// Get current filepointer, save it till later
		old_fp=f_tell(&fil);			
		
		// Keep reading bytes until we hit a block beggining byte
		do
		{
			f_read(&fil,&last_byte,1,&read);
		}
		while(!f_eof(&fil) && (CASBlockBegin != last_byte));
		
		// If we are not at the end of file, read a filename block and
		// copy filetype to globalData[0]
		if (!f_eof(&fil))
		{
			f_read(&fil,&FNBlock,sizeof(FNBlock),&read);
			globalData[0]=FNBlock.FileType;
		}
		// Restore old filepointer.
		f_lseek(&fil,old_fp);
	}

	WriteStatus(STATUS_COMPLETE);
	ClearBusy();	
}


#define MK_WORD(x,y) ((WORD)(x)<<8|(y))

// Read Eeprom
#define COM_RE MK_WORD('R','E')

// Write Eeprom
#define COM_WE MK_WORD('W','E')


void wfnExecuteArbitrary(void)
{
   if (globalAmount == 0 && globalDataPresent == 0)
   {
      WriteStatus(STATUS_ERROR | ERROR_NO_DATA);
	  ClearBusy();
      return;
   }

   switch (LD_WORD(&globalData[0]))
   {
   case COM_RE: // read eeprom
      {
         // globalData[2] = start offset, [3] = count

         WORD start = (WORD)globalData[2];
         WORD end = start + (WORD)globalData[3];

         WORD i, n = 0;
         for (i = start; i < end; ++i, ++n)
         {
            globalData[n] = ReadEEPROM(i);
         }

         WriteStatus(STATUS_COMPLETE);
  	     ClearBusy();
      }
      break;

   case COM_WE: // write eeprom
      {
         // globalData[2] = start offset, [3] = count

         WORD start = (WORD)globalData[2];
         WORD end = start + (WORD)globalData[3];

         WORD i, n = 4;
         for (i = start; i < end; ++i, ++n)
         {
            WriteEEPROM(i,globalData[n]);
         }

         WriteStatus(STATUS_COMPLETE);
 	     ClearBusy();
      }
      break;
   }
}
