`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    15:57:34 06/29/2011 
// Design Name: 
// Module Name:    Main 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module Main(
	// 6809 side
    input [15:0] Addr,		// Dragon address bus
    inout [7:0] Data,		// Daragon data bus
    input E,				// E clock
	input Q,				// Q clock
    input RW,				// Read/Write
    input CSEL,				// Cart rom select 
    input P2,				// P2/SCS acrive from $ff40-$ff5f
    input Reset,			// System reset
	output DSD,				// Device select disable
    output CART,			// Cart FIRQ
    output NMI,				// Non maskable interupt
    output HALT,			// Halt CPU
    
	// AVR side 
    inout [7:0] AData,		// AVR data bus
    input [1:0] AAddr,		// AVR data bus
    input nARD,				// AVR read strobe
    input nAWR,				// AVR write strobe
    input AVRBusy,			// AVR busy when 1
	output AInt,			// Interupt to AVR
	
	// Memory control
    output nRD,				// Read strobe
    output nWR,				// Write strobe
    output nROMCS,			// ROM chip select
    output nRAMCS,			// RAM chip select
	
	// Snapshot button
	input	Snap,			// Snapshot button
	
	// I/O Port, may not be used.
    inout [7:0] FData		// FTDI data bus
    );

	// Memory Mapping bits

	reg		RamEnable;		// Should RAM be ebnabled in the $8000-$FEFF area ?
	reg		MapMode;		// Copy of the SAM map mode bit set by write to $FFDF, ceared by $FFDF

	// 6809 and AVR output data registers.
	
	reg [7:0] 	DragonToAVR;	// Data going from Dragon -> AVR
	reg [7:0] 	AVRToDragon;	// Data going from AVR -> Dragon
	reg [4:0]	AddrLatch;		// Latch of written address.
	reg 		DragonRW;		// Did the dragon read or write ?
	reg			DragonSAMBit;	// Was the last dragon access a SAM bit ?
	reg			HaltState;		// Should the CPU be halted whilst waiting for AVR ?
	reg			RamWP;			// Ram Write protect
	reg			FIRQEn;			// FIRQ routing enable

	// Handshake bits
	reg			DragonW_AVRR;	// Set by Dragon write, cleared by AVR read.
	reg			AVRW_DragonR;	// Set by AVR write, cleared by Dragon read.
	
// Tristate some signals for now....

//	assign CART			= 1'bz;
	assign NMI			= 1'bz;

//	assign Data[7:0]	= 8'bz;
//	assign AData[7:0]	= 8'bz;
//	assign FData[7:0]	= 8'bz;

	// Generate OE and WE signals, only do this if Reset is high ! 
	assign RD			= E & RW & Reset;
	assign WR			= E & Q & ~RW & Reset;
//	assign WR			= Q & ~RW & Reset;
	assign nRD			= ~RD;
	assign nWR			= ~WR | RamWP;

//
// Address decoding and ROM/RAM banking control.
// 	
	// SAM Map bit, we latch writes to the SAM map bit here
	assign MapClear		= (Addr==16'hFFDE) & WR;
	assign MapSet		= (Addr==16'hFFDF) & WR;
	
	always @(negedge Reset or posedge MapClear or posedge MapSet)
	begin
		if(Reset==1'b0) 
			MapMode	<= 1'b0;
		else if (MapClear)
			MapMode	<= 1'b0;
		else 
			MapMode <= 1'b1;
	end
	
	// RAM enable and write protection, note NOT cleared by reset.
	assign RamCTRL		= (Addr==16'hFF56) & WR;
	
	always @(negedge RamCTRL)
	begin
		RamEnable	<= Data[0];
		RamWP		<= Data[1];
		FIRQEn		<= Data[2];
	end
	
	// CART/FIRQ redirect Q to CART if enabled.
	assign CART		= FIRQEn ? Q : 1'bz;
		
	// Address renages
	assign RangeRAM		= ((Addr>=16'h8000) && (Addr<=16'hfeff));
	assign RangeROM		= ((Addr>=16'hC000) && (Addr<=16'hfeff));
	
	// Enable ram either when writing to the RAM range or when reading from the range with 
	// RamEnable set, this way writes always go to the RAM.
	// Do not enable the RAM in MapMode 1 as onboard RAM enabled then.
	assign RAMCS		= ((RangeRAM & WR) | (RangeRAM & RamEnable)) & ~MapMode;
	assign nRAMCS		= ~RAMCS;

	// Enable ROM if we are in map mode 0 and RamEnable is false.
	assign ROMCS		= RangeROM & ~RamEnable & ~MapMode;
	assign nROMCS		= ~ROMCS;

	// Disable internal device selection only when our ram is enabled.
	assign DSD			= nRAMCS;
//
// AVR<-->Dragon data exchange
//
	assign DragonIO		= ((Addr>=16'hff50) && (Addr<=16'hff53));
	assign DragonIORD	= DragonIO & RD;
	assign DragonIOWR	= DragonIO & WR;
	
	assign AVRStatus	= (Addr == 16'hff54);		// AVR status at $FF54
	assign AVRStatusRD	= AVRStatus & RD;

//	assign SAMBitsAddr	= ((Addr>=16'hFFC0) && (Addr<=16'hFFDF));
//	assign SAMBitsWR	= SAMBitsAddr & WR;
	
	assign AddrLatchEN	= DragonIOWR; // | SAMBitsWR;
	assign AInt			= DragonIORD | DragonIOWR; // | SAMBitsWR;

	// Capture the bottom 5 address lines on a write by the Dragon to our IO registers
	// or to one ofg the SAM registers
	always @(posedge AddrLatchEN or negedge Reset)
	begin
	  if(!Reset)
	  begin
	    AddrLatch[4:0] <= 5'b00000;
		//DragonSAMBit <= 1'b0;
	  end
	  else
	  begin
	    AddrLatch[4:0] <= Addr[4:0];
		//DragonSAMBit <= SAMBitsWR;
	  end
	end

	// Latch read or write
	always @(posedge AInt)
	begin
	  DragonRW <= RW;
	end

	// When the AVR reads give it the Dragon data / latched address
	wire [7:0]	AVRDataOut;
//	assign 	AVRDataOut[7:0]	= AAddr[0] ? { DragonSAMBit,DragonRW,1'b0,AddrLatch[4:0] } : DragonToAVR[7:0];
	assign 	AVRDataOut[7:0]	= AAddr[0] ? { 1'b0,DragonRW,1'b0,AddrLatch[4:0] } : DragonToAVR[7:0];
	assign	AData			= nARD ? 8'bz : AVRDataOut;
	
	// When the Dragon reads give it the AVR data / status
	wire [7:0] 	DragonDataOut;
	assign 	DragonDataOut[7:0]		= AVRStatus ? {5'b0,AVRW_DragonR,DragonW_AVRR,AVRBusy} : AVRToDragon;
	assign 	Data					= (DragonIORD | AVRStatusRD) ? DragonDataOut : 8'bz;

	// Latch Dragon to AVR reg on Dragon write
	always @(negedge DragonIOWR)
	begin
	  DragonToAVR <= Data;
	end
	
	// Latch AVR to Dragon on AVR write
	always @(negedge nAWR)
	begin
	  AVRToDragon <= AData;
	end
	
	// Handshake lines.
	// DragonW_AVRR set by a write from the Dragon, cleared by a read by the AVR.
	// Cleared on reset.
	always @(negedge Reset or posedge DragonIOWR or negedge nARD)
	begin
		if(!Reset)
			DragonW_AVRR <= 1'b0;
		else if(DragonIOWR)
			DragonW_AVRR <=  1'b1;
		else
			DragonW_AVRR <= 1'b0;
	end

	// AVRW_DragonR set by a write from the AVR, cleared by a read by the Dragon.
	// Cleared on reset.
	always @(negedge Reset or posedge DragonIORD or negedge nAWR)
	begin
		if(!Reset)
			AVRW_DragonR <= 1'b0;
		else if(DragonIORD)
			AVRW_DragonR <=  1'b0;
		else
			AVRW_DragonR <= 1'b1;
	end
	
	// Haltstae whilst reading sam bits.
	
	//assign HaltSet		= SAMBitsWR;
	//assign HaltClear	= (~nARD & AAddr[1]) | ~Reset;
	//always @(posedge HaltSet or posedge HaltClear)
	//begin
	//	if(HaltSet)
	//		HaltState <= 1'b1;
	//	else 
	//		HaltState <= 1'b0;
	//end
	
	//assign Halt = HaltState ? 1'b0 : 1'bz;
	assign HALT = 1'bz;
endmodule
