;
; cmds_direct.asm : commands for loading and saving direct to the disk.
;

;
; MSave, same format as DragonDos save, files format compatible.
;
; MSave "filename"
; MSave "Filename",start,end,entry
;

CmdMSave
		pshs	u				; Make temp stack frame
		leas	-FileHeadLen,s
		leau	,s				; U=S
		
		lbsr	GetSendFileName		; Get filename send to MMC
		
        jsr     <$A5			; get next char
		beq		CmdSaveBasic	; nope : it's basic save as such
		jsr		<BasChrGet		; yes : eat it
		
		bsr		GetCommaThen16	; Get start addr
		stx		HdrLoad,u
		
		bsr		GetCommaThen16	; Get end addr
		tfr		x,d
		cmpx	HdrLoad,u		
		lbcs	BasFCError
		
		subd	HdrLoad,u		; Convert to length
		lbmi	BasFCError		; ?FC error if -ve
		addd	#1				; fixup 
		std		HdrLen,u		; Save length
		
		bsr		GetCommaThen16	; Get entry address
		stx		HdrExec,u
		
		lda		FTypeBin		; Set as binary
		
		bra		CmdSaveCommon	; Go save it
		
CmdSaveBasic
		ldx		<BasStartProg	; get start of program
		stx		HdrLoad,u
		
		ldd		<BasVarSimpleAddr	; get end of program
		subd	<BasStartProg	; calculate length
		std		HdrLen,u
		
		LDX     #BasFCError		; Exec address = FC Error
		stx		HdrExec,u

		lda		FTypeBas		; Set as basic
		
CmdSaveCommon
		sta		HdrType,u		

		lda		#$55			; Save ID bytes
		sta		HdrID55,u
		coma
		sta		HdrIDAA,u
		
		leax	,u				; Get address of header
		lbsr	MMC_SaveFile	; Save the file
		
		lbsr	MMC_CloseFile	; close the file
		
CmdMSaveExitOK
		clra
CmdMSaveExit
		leas	FileHeadLen,s	; cleanup stack frame
;		leas	,u				; restore SP
		puls	u,pc			; restore / return
		

GetCommaThen16
		jsr     >VarCKComma		; Check for comma
        jmp     >VarGet16Bit	; returned in X, FCError if invalid
		
		
;
; MLoad, load a DragonDos format compatible file.
;
; Syntax :
;	LOAD filespec
;	LOAD filespec,offset
;

CmdMLoad
		rts
		
		ifeq	1
		
		pshs	u				; Make temp stack frame
		leas	-FileHeadLen,s
		leau	,s				; U=S
		
		lbsr	GetSendFileName		; Get filename send to MMC
		leax	,u				; Get address for file header
		ldb		#FileHeadLen	; Read file header bytes
		lbsr	MMC_ReadFileBlock	; go read it
		
		lda		HdrID55,u		; Check marker bytes
		cmpa	#$55			
		bne		CmdMLoadError		
		
		lda		HdrIDAA,u
		cmpa	#$aa
		bne		CmdMLoadError		
		
		lda		HdrType,u		; Get filetype
		cmpa	#FTypeBas		; Basic file ?
		beq		CmdMLoadBas		; yep : load it
		
		cmpa	#FTypeBin		; Binary file ?
		bne		Cmd<LoadError	; nope, error
	
CmdMLoadBin

CmdMLoadExit
		leas	FileHeadLen,s	; cleanup stack frame
		clra
		puls	u,pc			; restore / return

CmdMLoadBas
		bra		CmdMloadExit

CmdMLoadError
		jmp		BasFCError		; Error !
		
		endc