;
; Debug Module, USB->serial output via FTD245.
;

LF	EQU	$0A
CR	EQU	$0D

;
; Wait for FT245 TxE bit to be active, then transmit byte in a
;

FTWaitTx:
		pshs	a				; save a
FTWaitTxLoop
		lda		FTCTRL			; Read status
		bita	#FTTxE			; Ready for our data ?
		bne		FTWaitTxLoop	; Nope keep waiting

		puls	a				; restore byte to send
		sta		FTData			; Send the byte
		rts
	
FTWaitTXCooked
		cmpa	#LF				; Line feed ?
		beq		FTSendCRLF		; Yep send CR+LF
		cmpa	#CR				; Carridge return  ?
		beq		FTSendCRLF		; Yep send CR+LF
		bra		FTWaitTX
	
FTSendCRLF
		pshs	a
		lda		#CR				; send CR
		bsr		FTWaitTX
		lda		#LF				; send LF
		bsr		FTWaitTX
		puls	a,pc
	
;
; Wait for FT245 RxF bit to be active, then recieve a byte into a
;

FTWaitRx:
FTWaitRxLoop
		lda		FTCTRL			; Read status
		bita	#FTRxF			; Byte ready for us ?
		bne		FTWaitRxLoop	; Nope keep waiting
		lda		FTData			; Else get the data byte
		rts
	
;
; Transmit an ASCIIZ string at X
;

FTSendStrX:
		lda		,X+				; Get a byte at X
		tsta					; test and set flags
		beq		FTSendStrXExit	; Zero ? = end of string, yes : exit
		bsr		FTWaitTx		; Send it
		bra		FTSendStrX		; Do next
FTSendStrXExit
		rts

FTSendI
		pshs	a
		lda		#$49			; Send I
		bra		FTSendIO
	
FTSendO
		pshs	a
		lda		#$4F			; Send O
	
FTSendIO
		bsr		FTWaitTx		
		puls	a
		bsr		FTSendHexByte	; send as hex
		bsr		FTWaitTx		; send char
		bra		FTSendCRLF

	
FTSendHexByte
		pshs	a
	
		lsra					; Move msn to lsn
		lsra	
		lsra	
		lsra	
		bsr		FTSendHexDigit	; send msn
	
		lda		,s				; retrive digit
		bsr		FTSendHexDigit	; send lsn
	
		lda		#$20			; send space
		bsr		FTWaitTx		; send it
	
FTSendHexByteExit	
		puls	a,pc

FTSendHexDigit
		bsr		GetHexNibbleA	; Get hex nibble in A
		bsr		FTWaitTx		; send char
		rts
	
GetHexNibbleA
		pshs	x
		anda	#$0F			; mask out bytes
	
		leax	HexDigits,pcr	; Point to digits
		leax	a,x				; point to needed digit
		lda		,x				; **GET** hex digit !
		puls	x,pc
	
	
HexDigits
		fcc		/0123456789ABCDEF/
	
FTSendHexByteEOL
		bsr		FTSendHexByte	; Send hex byte
		bra		FTSendCRLF		; send CR+LF
		

		
