;
; DragonMMC system ROM,
;
; 2011-06-13, Phill Harvey-Smith.
;
; Fixed for assembly with "mamou" cross-assembler,
; from NitrOS9 cocotools.
;


;
; These are defined by the makefile
;
;DragonDos		EQU		1	; Define this if compiling for Standard DragonDos Cart.
;DragonAlpha	EQU		1	; Define this if compiling for Dragon Alpha.
;RSDos			EQU		1	; Define this if compiling for the RS-DOS cart.

; Set to > 0 to enable debugging 
DEBUG		EQU	0

Dragon		EQU	1


SCSMODE		EQU	1

			ifp1
			use		cpudefs.asm
			use		dgndefs.asm
			use		dosdefs.asm
			use		romdefs.asm
			use		atmmc2def.asm
			use		DragonMMCdef.asm
			endc
					
			ORG     $C000


; Disk controler ID, if a cartrage starts with the chars 'DK', then the basic rom routines
; will do a JMP to $C002 to init the cartrage.

DC000   FCC     /DK/				

; Jump table.
LC002   LBRA    DosInit
Util_SafeCopy
		LBRA	DO_SafeCopy		; Jump to safe copy routine	
Util_RamCopy
		LBRA	DO_RamCopy
		
;
; Init Dos
; 
				
DosInit
;		LDX     #DosAreaStart	; Point to bottom of dos vars area	
;        TFR     X,Y			
;LC02F   CLR     ,X+				; Clear a byte, increment X
;        LEAY    -1,Y			; decrement counter
;        BNE     LC02F			; loop again if more to do
;	
; X now points to the top of the dos variable area
;
;        TFR     X,D
;        TFR     A,B
;        ADDB    #$18
;        STB     <BasStartProg	; Setup new begining of basic
;        JSR     >$AA87
;        LDA     <GrDisplayStartAddr	; Adjust graphics ram pages
;        ADDA    #$06
;        STA     <GrLastDisplayAddr
	
;
; Init various low ram stuff, inturrupt vectors, basic stub etc
; 
	
        LDX     #RamVarPointer	; Point to rom copy of data to copy
LC049   LDB     ,X+				; Get byte count byte
        BEQ     LC054			; zero= end of table, exit copy
        LDU     ,X++			; Get destination address
        JSR     >UtilCopyBXtoU	; do copy
        BRA     LC049			; do next

LC054   LDX     #NewUsrVec		; Adjust usr vector base
        STX     <BasUSRTableAddr	
        LDU     #BasFCError		; Setup the 10 usr vectors to point to BasFCError
        LDB     #$0A			; do 10
LC064   STU     ,X++			; setup vector
        DECB					; decrement count
        BNE     LC064			; loop again if more to do
		
        LDX     #ResetVector	; Setup new reset vector
        STX     <IndVecReset
		lda		#$55			; Mark reset vector valid
		sta		<WarmStartFlag
		
;		lbsr	InitNMI			; Init NMI code
		
        ANDCC   #$AF			; reenable inturrupts
	
        LDX     #BasSignonMess ; Print staandard Basic signon message
        JSR     >TextOutString
        
        LDX     #SignonMess-1
        JSR     >TextOutString
		lbsr	TestAutoBoot	; test for auto-boot, does not return if boots
        JMP     >BasCmdMode		; Jump to normal basic command mode
	
;
; Dispatch routines for new functions/commands
;

CmdDispatch   
		CMPA    #$FF
        LBEQ    BasSNError
        SUBA    #$CE
        BPL     LC67D
LC67A   JMP     >BasSNError

LC67D   CMPA    #$1A
        BCC     LC687
        LDX     #CommandDispatchTable
        JMP     >$84ED

LC687   JMP     [>$0137]

FuncDispatch   
		SUBB    #$44
        BPL     LC691
        BRA     LC67A

LC691   CMPB    #$0E
        BCC     LC69D
        LDX     #FunctionDipatchTable
        JSR     [B,X]
        JMP     >$8874

LC69D   JMP     [>$013C]


;
; New reset vector
;

ResetVector   
		NOP					; Main ROM checks for reset->NOP
        CLRA				; Reset DP=0
        TFR     A,DP		
        LDA     #$35		; Re-enable NMI
        STA     PIA0CRB
        
		JMP     >$B44F		; Jump back to Main ROM reset routine

RamVarPointer   
; New basic dispatch stub
	
        FCB     $1E			; No bytes
        FDB     $012A		; address to copy

;
; Stub for our new commands
;
        FCB     CMDTableLen		; No of commands
        FDB     CmdNames		; command list
        FDB     CmdDispatch		; command dispatch routine
        FCB     FuncTableLen	; No of functions
        FDB     FuncNames		; function list
        FDB     FuncDispatch	; function dispatch table

;
; New null terminating stub
;
        FCB     $00				; No of commands
        FDB     $0000			; Null list
        FDB     BasSNError		; command dispatch 
        FCB     $00				; no of functions
        FDB     $0000			; function dispatch
        FDB     BasSNError
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
        FCB     $00
	
        FCB     $00			; No bytes : terminate
        FCB     $00
;
; Command routine pointers
; 
CommandDispatchTable
		FDB		CmdMCas	
		FDB		CmdMMirror
		FDB		CmdMRMount
		FDB		CmdMWMount
		FDB		CmdMUNMount
		FDB		CmdHelp
		FDB		CmdRamBoot
		FDB		CmdCat
		FDB		CmdMDelete
		FDB		CmdMSave
		FDB		CmdMLoad
		FDB		CmdCartLoad
		FDB		CmdTapeRun
		FDB		CmdCWD
		FDB		CmdSetCFG

; Note no spaces must appear in the EQU below!
CMDTableLen		EQU	(*-CommandDispatchTable)/2


FunctionDipatchTable   
		FDB     0
        FDB		FuncCWD
		FDB		FuncFindFirst
		FDB		FuncFindNext
		FDB		FuncCFType
		FDB		FUNCGetCFG
		
; Note no spaces must appear in the EQU below!
FuncTableLen		EQU	(*-FunctionDipatchTable)/2
	
FuncNames   
		FCS		/VER/
		FCS		/DIR$/
		FCS		/FINDFIRST$/
		FCS		/FINDNEXT$/
		FCS		/CFTYPE/
		FCS		/MGETCFG/
	
CmdNames   
		FCS		/MCAS/
		FCS		/MMIRROR/
		FCS		/MRMOUNT/
		FCS		/MWMOUNT/
		FCS		/MCLOSE/
		FCS		/HELP/
		FCS		/RAMBOOT/
		FCS		/CAT/
		FCS		/MDELETE/
		FCS		/MSAVE/
		FCS		/MLOAD/
		FCS		/MCARTLOAD/
		FCS		/MTAPERUN/
		FCS		/CWD/
		FCS		/MSETCFG/

		use		message.asm
;
; Get string from command into temp var.
; Exits :
;	x = pointer to string data
;	b = length of string
;

GetCmdStrXB
        JSR     >VarGetStr		; get string into temp variable
		JSR		>BasGetStrLenAddr	; Get pointer in x len in b
		rts

;
; Get a filename and send to MMC
;

GetSendFileName
		bsr		GetCmdStrXB		; Get string from basic

		lbsr	MMC_SendName	; Send name to MMC
		rts
		
;
; Get a filename and send to MMC floowed by command in a
;
		
CmdSendFilenameCmd
		pshs	a
		bsr		GetSendFileName	; get filename and send it
		
		puls	a				; Restore read or write
		lbsr	MMC_SendCmd		; Open file
		lbsr	CheckError		; Bomb out on error
		
		clra
		rts

CkComma
		ldb		#$2c			; comma character
		cmpb	[BasAddrSigByte]		
CkCommaExit
		rts


CheckError
	cmpa	#64							; Error ?
	bhi		FlagError					; Yes : flag it
	rts
	
FlagError
	lbsr	CONWriteHexByte				; Write to screen as hex
	lbsr	CON_EOL
	jmp		BasFCError
	
;
; Copy all the ROM to ram.
;

SwitchToRAM
		LEAX	ROMCOPY,PCR
		LDY		#$600
		LDU		#ROMCOPYLEN
	
COPYLOOP
		LDA		,X+
		STA		,Y+
		LEAU	-1,U
		CMPU	#$0
		BNE		COPYLOOP
		JMP 	$600
	

ROMCOPY	
		ORCC	#$50
		LDX		#$8000
	
LOOP	
		CLR		$FFDE
		LDA		,X
		CLR		$FFDF
		STA		,X+
		CMPX	#$FF00
		BNE		LOOP

		ANDCC	#$AF
		CLR		$FFDF
		RTS
	
ROMCOPYEND
		NOP
	
ROMCOPYLEN		EQU	ROMCOPYEND-ROMCOPY

CmdMMirror
		rts

CmdHelp
        leax    SignonMess-1,pcr		; Print signon message including compile date
        JSR     >TextOutString
		
		leax	IFVerMess-1,pcr			; print IF ver message
		JSR     >TextOutString
		
		lda		#CMD_GET_FW_VER			; Get firmware version
		lbsr	MMC_SendCmd				
		
		lbsr	CON_NdotN				; Write msn.lsn
		lbsr	CON_EOL
		
		clra
		rts

; Command includes
		use		cmds_tape.asm
		use		cmds_ram.asm
		use		cmd_cat.asm
		use		cmd_delete.asm
		use		cmds_direct.asm
		use		cmds_cart.asm
		use		cmds_config.asm
	
; Driver function includes
		use		mmc_func.asm
		use		console.asm
		use		util.asm

;********************************
;** PLACE NO CODE BELOW HERE ! **
;********************************

		zmb		$Cffd-*
	
		org		$Cffd
		FCC		/PHS/
		
;DE000   END
