;
; MMC functions.
;
; 2011-06-13 Phill Harvey-Smith.
;

;
; Note some routines make calls / returns into the Dragon and CoCo roms.
; Please remember to define them so they assemble correctly on *BOTH* 
; machines.
; 

;
; **** NOTE ****
;
; Send CMD_INIT_READ with MMC_sendCmdRaw *NOT* MMC_SendCmd
;


		ifne	Dragon
CasRDCksum		EQU		$B972				
CasWRCksum		EQU		$B9C9
		else
CasRDCksum		EQU		$A73B
CasWRCksum		EQU		$A824
		endc

;
; Send Command
;

MMC_SendCmd
		bsr		MMC_SendCmdRaw			; send the command

		lda		#STATUS_FLAG_WRITTEN	; Written flag
		anda	D_STATUS_REG			; is it set ?
		beq		MMC_SendCmdNoResult		; no clear a and return
		lda		D_CMD_REG				; Get status (if any)
		rts								; return
		
MMC_SendCmdNoResult
		clra							; flag no error
		rts
		
;
; Send a command but don't read a result.
;

MMC_SendCmdRaw
		sta		D_CMD_REG				; send the command
		bsr		MMC_WaitRead			; Wait for AVR to read it
		bsr		MMC_WaitNotBusy			; wait command completion
		rts
		
;
; Init buffer write
;
MMC_InitSendBBytes
		pshs	a
		lda		#CMD_INIT_WRITE			; Write bytes
		bsr		MMC_SendCmd				; send the command
		puls	a
		rts
;
; Init write buffer and send B bytes from X to MMC 
;

MMC_StartSendBBytes
		bsr		MMC_InitSendBBytes
;
; Send B bytes pointed to by X to the WRITE_DATA_REG
; if b=0, send 256 bytes.
;
MMC_SendBBytes
		pshs	d,x
MMC_SendBBytes2
		lda		,x+						; get a byte

		bsr		MMC_WaitPutRead			; send byte, wait for AVR to read it

		decb							; decrement count
		bne		MMC_SendBBytes2			; more : loop again

		puls	d,x,pc					; restore and return
		

;
; Init buffer and write B bytes from X, and terminate with 0 byte 
;	
; MMC_Sendname2 differs from MMC_Sendname, in that it assumes that a write 
; bytes command has already been written, for example if parameters have been 
; written before the filename
;
MMC_SendBBytesZero
MMC_SendName
		bsr		MMC_InitSendBBytes		; start sending bytes

MMC_SendName2
		pshs	a
		bsr		MMC_SendBBytes			; Send the bytes
		
		lda		#0						; Send terminator
		bsr		MMC_WaitPutRead			; send byte, wait for AVR to read it
		puls	a,pc					; restore and return
		
;
; MMC_WaitNotBusy, waits for the AVR busy flag to be reset.
;
MMC_WaitNotBusy
		lda		D_STATUS_REG			; Busy flag
		anda	#STATUS_FLAG_BUSY		; is it set ?
		bne		MMC_WaitNotBusy			; yes : keep waiting
		rts

;
; MMC_WaitPutLatchRead Send a byte in a to the AVR LATCH_REG and wait for it to read it
;
MMC_WaitPutLatchRead
		sta		D_LATCH_REG				; write the data
		bra		MMC_WaitRead			; and wait for it to be read
;
; MMC_WaitPutRead Send a byte in a to the AVR DATA_REG and wait for it to read it
;

MMC_WaitPutRead 
		sta		D_WRITE_DATA_REG		; send it, and fall through.....

;
; MMC_WaitRead : waits for the AVR to read byte
;
MMC_WaitRead
		lda		D_STATUS_REG
		anda	#STATUS_FLAG_READ		; Has it been read yet ?
		bne		MMC_WaitRead			; no : keep waiting
		rts

;
; MMC_WaitWritten, waits for the AVR to write a byte
;
MMC_WaitWritten
		lda		D_STATUS_REG
		anda	#STATUS_FLAG_WRITTEN	; Written flag is it set ?
		beq		MMC_WaitWritten			; no : keep waiting
		rts

;
; MMC_WaitGetWritten, wait for the AVR to write a byte, return it in a
;
MMC_WaitGetWritten
		bsr		MMC_WaitWritten			; Wait for byte
		lda		D_READ_DATA_REG			; get byte
		rts
;
; MMC_WaitGetWrittenReturn, wait for the AVR to write a byte, return it to basic
;

MMC_WaitGetWrittenReturn
		bsr		MMC_WaitWritten			; Wait for byte
		ldb		D_READ_DATA_REG			; get byte
		jmp		VarAssign8Bit			; return it to basic

;
; MMC_CloseFile : close current file
;		
MMC_CloseFile
		lda		#CMD_FILE_CLOSE			; Close the file
		lbsr	MMC_SendCmd				; send the command
		rts

		
;
; MMC_ReadFByte : Read a byte from a file and return it in a
;
MMC_ReadFByte
		lda		#1						; One byte
		bsr		MMC_ReadFCommon
		bra		MMC_WaitGetWritten		; get the byte, returned in a

MMC_ReadFCommon
		sta		D_LATCH_REG				; Send to AVR
		bsr		MMC_WaitRead	
		
		lda 	#CMD_READ_BYTES			; Read bytes from file
		bsr		MMC_SendCmd				; send the command
		
		lda		#CMD_INIT_READ			; Read the byte
		bsr		MMC_SendCmdRaw			
		rts
		
;
; Read a block from open cas file, length in 
; Entry :
;	CasBlockLen 	= size of block in bytes $00..$FF
;	X				= Pointer to buffer to recieve bytes
;
; Exit :
;	X				= Updated to point to byte after last byte read.
;	a				= error code, $00=none, $01=Csum error, $02=invalid dest addr
;	CasCkSum		= Checksum of block
;	
;

MMC_ReadFBlock
		bsr		MMC_ReadFileBlockCas	; read the block
		jmp		CasRDCksum				; and back to the rom for the checksum
;		
; Read a block pointed to by X from MMC 
;
; Entry :
;	b				= size of block in bytes $00..$FF
;	X				= Pointer to buffer to recieve bytes
;
; Exit :
;	X				= Updated to point to byte after last byte written.
;

MMC_ReadFileBlock
		stb		<CasBlockLen		; Save block length
		stb		<CasIOErrorCode		; running count
		
MMC_ReadFileBlockCas
		lda		<CasBlockLen		; get length of block
		bsr		MMC_ReadFCommon		; Set the bytes
		
MMC_ReadFBlockLoop
		lbsr	MMC_WaitGetWritten	; get the byte, returned in a
		sta		,x					; save in buffer
		cmpa	,x+					; Did it save ok ? 
		bne		MMC_ReadFBlockError	; Yep eat rest of read and return error
		adda    <CasCkSum			; add read byte to checksum
        sta    	<CasCkSum			
        dec     <CasIOErrorCode		; decrement byte count
		bne		MMC_ReadFBlockLoop	; Keep going if more bytes

		rts
	
		suba	<CasCkSum			; check against calculated checksum
		bne		MMC_ReadFBlockExit
	
		lda		#$01				; Flag error
	
MMC_ReadFBlockExit
		sta		<CasIOErrorCode		; Save error code
		rts	

MMC_ReadFBlockError
		lda		#$02				; Flag error
		bra		MMC_ReadFBlockExit	
		
		
		
;
; Write a byte to open cas file, from a
;
	
MMC_WriteFByte
		pshs	a
		lda		#CMD_INIT_WRITE		; Initiialise write
		lbsr	MMC_SendCmd			; send command
		lda		,s					; peek a back off stack 
		
		lbsr	MMC_WaitPutRead 	; send byte to AVR
		
		lda		#1					; Write 1 byte
		sta		D_LATCH_REG			; tell AVR
		lbsr	MMC_WaitRead		; wait for it to be read
		
		lda		#CMD_WRITE_BYTES	; Write bytes
		lbsr	MMC_SendCmd			; send command
		
		puls	a,pc				; restore and return
		
;
; Write a block pointed to by X to MMC (tape emulation)
;
; Entry :
;	CasBlockLen 	= size of block in bytes $00..$FF
;	CasIOErrorCode	= CasBlockLen
;	X				= Pointer to buffer to recieve bytes
;
; Exit :
;	X				= Updated to point to byte after last byte written.
;

MMC_WriteFBlock
		bsr		MMC_WriteFileBlockCas	; Write it
		jmp		CasWRCksum				; Jump back to rom

;		
; Write a block pointed to by X to MMC 
;
; Entry :
;	b				= size of block in bytes $00..$FF
;	X				= Pointer to buffer to recieve bytes
;
; Exit :
;	X				= Updated to point to byte after last byte written.
;

MMC_WriteFileBlock
		stb		<CasBlockLen		; Initialise block len / count
		stb		<CasIOErrorCode
		
MMC_WriteFileBlockCas
		lda		#CMD_INIT_WRITE		; Initiialise write
		lbsr	MMC_SendCmd			; send command
MMC_WriteFBlockLoop
;		com		$401
		lda		,x+					; get byte to write
		lbsr	MMC_WaitPutRead 	; send byte to AVR
	
		dec     <CasIOErrorCode		; Decrement count
		bne		MMC_WriteFBlockLoop	; More ? yep : keep going
		
		lda		<CasBlockLen		; get length of block
		sta		D_LATCH_REG			; send to AVR
		lbsr	MMC_WaitRead		; wait for it to be read
		
		lda		#CMD_WRITE_BYTES	; Write it to the file
		lbsr	MMC_SendCmd			; send it
		rts	

;
; MMC_SaveFile, save to the open file 
; 
; Entry :
;	u				= Pointer to the file header for the file to save.
;

MMC_SaveFile
		ldb		#FileHeadLen		; Header length
		leax	,u					; point to header
		bsr		MMC_WriteFileBlock	; Write the block

		ldx		HdrLoad,u			; Get load address

MMC_SaveFileLoop	
		ldd		HdrLen,u			; Get file length
		cmpd	#$100				; > 256 bytes ?
		blo		MMC_SaveTail		; no : last block

		subd	#$100				; decrement length
		std		HdrLen,u			; update
		
		clrb						; flag 256 bytes
		bsr		MMC_WriteFileBlock	; Write the block
		bra		MMC_SaveFileLoop	; Do next block	
		
MMC_SaveTail
		tstb						; any more bytes ?
		beq		MMC_SaveFileExit	; nope : exit
		bsr		MMC_WriteFileBlock	; Write the block
				
MMC_SaveFileExit
		clra	
		rts
		
;
; MMC_LoadFile, load from the open file
;
; Entry :
;	u				= Pointer to the file header for the file to load.
;

MMC_LoadFile
		ldx		HdrLoad,u			; get load address
		
MMC_LoadFileLoop
		ldd		HdrLen,u			; Get length remaining
		cmpd	#$100				; > 256 bytes ?
		blo		MMC_LoadTail		; no : last block
		
		subd	#$100				; decrement remaining length
		std		HdrLen,u			; update
		
		clrb						; flag 256 bytes
		lbsr	MMC_ReadFileBlock	; read next block
		bra		MMC_LoadFileLoop	; do next block
		
MMC_LoadTail
		tstb						; any more bytes ?
		beq		MMC_LoadFileExit	; nope : exit
		lbsr	MMC_ReadFileBlock	; load last block
		
MMC_LoadFileExit
		clra
		rts

;
; Send LBA to AVR
;
MMC_SendLBA
		pshs	u,d
		lbsr	MMC_InitSendBBytes	; Begin sending bytes
		
		lbsr	MMC_WaitPutRead		; send drive id it
		
		exg		u,d					; get LSW of LBA
		exg		a,b					; move lsb into a
		
		lbsr	MMC_WaitPutRead		; send LSB of LSN
		
		exg		a,b					; get next byte of LBA
		
		lbsr	MMC_WaitPutRead		; send it
		
		exg		d,u					; retrieve MSB
		
		exg		a,b					; move lsb into a
		lbsr	MMC_WaitPutRead		; send it

		clra						; msb of LSN always 0
		lbsr	MMC_WaitPutRead		; send it

		lda		#CMD_LOAD_PARAM		; Tell AVR
		lbsr	MMC_SendCmd			; send command
		
		puls	u,d,pc				; restore / return
		
;
; MMC_ReadDOSSec,  Read an emulated dos sector
; MMC_ReadDOSSec2  Read just the first two bytes, used by 'BOOT'
;
; Entry :
;	A	= drive id
; 	B	= MSB of LSN
;	U	= LSB of LSN
;	X	= buffer
; For floppy images, B should always be zero !
;

MMC_ReadDOSSec
		bsr		MMC_ReadDOSSecInit	; init the read
		bcs		MMC_ReadDOSSecExit	; Error : exit
		
		clrb						; init count
		bra		MMC_ReadDOSSecLoopB	; Go read the data

MMC_ReadDOSSec2
		bsr		MMC_ReadDOSSecInit	; init the read
		bcs		MMC_ReadDOSSecExit	; Error : exit
		ldb		#$FE				; init count, reads 2 bytes

	
MMC_ReadDOSSecLoopB
;		lbsr	ConWriteHexX
MMC_ReadDOSSecLoop		
		lbsr	MMC_WaitGetWritten	; wait for byte
		com		$400
		sta		,x+					; save in buffer
		incb						; inc count
		bne		MMC_ReadDOSSecLoop	; not done, loop again

		clra
MMC_ReadDOSSecExit
		rts
		
MMC_ReadDOSSecError
		orcc	#FlagCarry			; flag error, code in a
		rts

MMC_ReadDOSSecInit

		bsr		MMC_SendLBA			; send LBA to AVR
		
		lda		#CMD_READ_IMG_SEC	; Read sector from AVR
		lbsr	MMC_SendCmd			; send command
		bmi		MMC_ReadDOSSecError	; if error handle it
		
		lda		#CMD_INIT_READ		; read the sector
		lbsr	MMC_SendCmdRaw		; send command
		rts

;
; MMC_WriteDOSSec,  Read an emulated dos sector
;
; Entry :
;	A	= drive id
; 	B	= MSB of LSN
;	U	= LSB of LSN
;	X	= buffer
; For floppy images, B should always be zero !
;

MMC_WriteDOSSec
		rts
		
;
; Get disk geometry of drive in DosLastDrive
;
; Entry : DosLastDrive contains drive to interigate
; Exit	: DosDxTracks and DosDxSecTrack set for drive
;		  Also returned in A=Tracks, B=Sec/Trk
;
; If an error occours, or a geometry block is not present
; defaults to standard single sided, 40 track.
;

MMC_GetGeom
		pshs	x
		lda		<DosLastDrive		; Pickup drive id
		deca						; make zero based

		ldx		#DosD0Online		; Point to tables for this drive
		leax	a,x

		lbsr	MMC_WaitPutLatchRead	; send the drive id		
		
		lda		#CMD_IMG_GET_GEOM	; get geometry
		lbsr	MMC_SendCmd			; send command
		
		lda		#CMD_INIT_READ		; Read the results
		lbsr	MMC_SendCmdRaw		; send command
		
		lbsr	MMC_WaitGetWritten	; get sec/track
		
		sta		DosSecTrkTblOfs,x	; save sec/trk it
		exg		a,b					; get sec / trk in b
	
		lbsr	MMC_WaitGetWritten	; get tracks
		sta		DosTracksTblOfs,x	; save tracks it
		
		puls	x,pc				; restore / return
