;
; cmds_tape.asm : Cassette emulation commands.
;

;
; Note some routines make calls / returns into the Dragon and CoCo roms.
; Please remember to define them so they assemble correctly on *BOTH* 
; machines.
; 
		ifne	Dragon
CloadMEntry		EQU		$A095
CloadEntry		EQU		$B6E1
		else
CloadMEntry		EQU		$A502
CloadEntry		EQU		$A4A3
		endc


FTBasic	equ		0
FTCode	equ		2

;
; Mount a cas file for input.
;
; syntax :
;	mrmount "filename"
;
CmdMRMount

		lda		#CMD_FILE_OPEN_READ	; Open file
		
CmdMMountCommon
		lbra	CmdSendFilenameCmd

;
; Mount a cas file for output.
;
; syntax :
;	mwmount "filename"
;

CmdMWMount
		lda		#CMD_FILE_OPEN_WRITE	; Open file
		bra		CmdMMountCommon			; rest same as rmount

;
; Unmount a cas file, closing it and flushing any output.
;

CmdMUNMount
		lbsr	MMC_CloseFile			; Close tape file
		lbsr	CheckError				; Check for error
		clra							
		rts

;
; Get the filetype of the next file on (emulated) tape
; 
; Syntax :
; 	X=CFTYPE


FuncCFType
		bsr		GetCFTypeA				; get cas filetype in A

		tfr		a,b						; move to b	
		jmp		VarAssign8Bit			; return it to basic


GetCFTypeA
		lda		#CMD_CAS_FTYPE			; Get file type
		lbsr	MMC_SendCmd				
		lbsr	CheckError				; Bomb out on error
		
		lda		#CMD_INIT_READ			; init read
		lbsr	MMC_SendCmdRaw
		lbsr	MMC_WaitGetWritten		; Wait for byte, and get it in a
		rts
		
;
; Enable tape emulatiom, open, and load a tape using cload/cloadm as needed.
;

CmdTapeRun
		lbsr	CmdMCasNormal			; boot into RAM mode
		lbsr	CmdMRMount				; Open the file
		bsr		GetCFTypeA				; get cas filetype in A

		cmpa	#FTBasic				; Basic program ?
		beq		CmdTapeRunLBas			; yes load it
		
		cmpa	#FTCode					; Machine code ?
		beq		CmdTapeRunCode			; yes load it
		
		jmp		BasFCError				; nope : error 

		
CmdTapeRunLBas
		CLR     <CasStatus
		leas	2,s
		
		jmp		CloadEntry				; cload the file

CmdTapeRunCode
		CLR     <CasStatus
		jmp		CloadMEntry				; cloadm the file
		