;
; cmds_direct.asm : commands for loading and saving direct to the disk.
;

;
; MSave, same format as DragonDos save, files format compatible.
;
; MSave "filename"
; MSave "Filename",start,end,entry
;

CmdMSave
		cmpa	#$4D			; check for 'M'
		pshs	a				; save next letter
		bne		CmdMSaveFname	; nope, skip
		JSR     <BasChrGet		; skip the M

CmdMSaveFname
		lbsr	GetSendFileName	; Get filename send to MMC
		puls	a

		pshs	u				; Make temp stack frame
		leas	-FileHeadLen,s


        cmpa	#$4D			; is it MSAVEM  
		bne		CmdSaveBasic	; nope : it's basic save as such
		
		leau	,s				; U=S
		
		bsr		GetCommaThen16	; Get start addr
		stx		HdrLoad,u
		
		bsr		GetCommaThen16	; Get end addr
		tfr		x,d
		cmpx	HdrLoad,u		
		lbcs	BasFCError
		
		subd	HdrLoad,u		; Convert to length
		lbmi	BasFCError		; ?FC error if -ve
		addd	#1				; fixup 
		std		HdrLen,u		; Save length
		
		bsr		GetCommaThen16	; Get entry address
		stx		HdrExec,u
		
		lda		#FTypeBin		; Set as binary
		
		bra		CmdSaveCommon	; Go save it
		
CmdSaveBasic
		leau	,s				; U=S
		ldx		<BasStartProg	; get start of program
		stx		HdrLoad,u
		
		ldd		<BasVarSimpleAddr	; get end of program
		subd	<BasStartProg	; calculate length
		std		HdrLen,u
		
		LDX     #BasFCError		; Exec address = FC Error
		stx		HdrExec,u

		lda		#FTypeBas		; Set as basic
		
CmdSaveCommon
		sta		HdrType,u		

		lda		#$55			; Save ID bytes
		sta		HdrID55,u
		coma
		sta		HdrIDAA,u
		
		lda		#CMD_FILE_OPEN_WRITE	; Open file
		lbsr	MMC_SendCmd		; Open file
		lbsr	CheckError		; Bomb out on error
		
		lbsr	MMC_SaveFile	; Save the file
		
		lbsr	MMC_CloseFile	; close the file
		
CmdMSaveExitOK
		clra
CmdMSaveExit
		leas	FileHeadLen,s	; cleanup stack frame
		puls	u,pc			; restore / return
		

GetCommaThen16
		pshs	u				; save u as often clobbered !
		jsr     >VarCKComma		; Check for comma
        jsr     >VarGet16Bit	; returned in X, FCError if invalid
		puls	u,pc			; restore / return
		
;
; MLoad, load a DragonDos format compatible file.
;
; Syntax :
;	MLOAD filespec
;	MLOAD filespec,new_start_addr
;

CmdMLoad
		bsr		DoCmdMLoad			; attempt to load
		tsta						; Error ?
		beq		CmdMLoadOK			; nope : just return
		jmp		BasFCError			; Yep : error 
		
CmdMLoadOK
		rts
		
DoCmdMLoad
		pshs	a,u					; save next character
		cmpa	#$41				; is it LOADA ?
		bne		CmdMLoadNoAuto		; nope : do nothing

		JSR     <BasChrGet			; skip the A
		
CmdMLoadNoAuto
		lbsr	GetSendFileName		; Get filename send to MMC

CmdMLoadAutoBoot
		leas	-FileHeadLen,s		; Make temp stack frame

		lda		#CMD_FILE_OPEN_READ	; Open file
		lbsr	MMC_SendCmd			; Open file
		cmpa	#64					; Error ?
		bhi		CmdMLoadError		; Yes : flag it

		leau	,s					; U=S

		leax	,u					; Get address for file header
		ldb		#FileHeadLen		; Read file header bytes
		lbsr	MMC_ReadFileBlock	; go read it
		
		lda		#$55				; Check marker bytes
		cmpa	HdrID55,u		
		bne		CmdMLoadError		
		
		coma
		cmpa	HdrIDAA,u
		bne		CmdMLoadError		
		
		lda		HdrType,u			; Get filetype
		cmpa	#FTypeBas			; Basic file ?
		beq		CmdMLoadBas			; yep : load it
		
		cmpa	#FTypeBin			; Binary file ?
		bne		CmdMLoadError		; nope, error

		ldx		HdrExec,u			; Get exec address
		stx		BasExecAddr			; set exec address
	
CmdMLoadBin
		lbsr	CkComma				; Comma ?
		bne		CmdMLoadBinDef		; no : use default addresses
		
		bsr		GetCommaThen16		; get new load address
		leau	,s					; U=S
		
		ldd		HdrExec,u			; get exec address
		subd	HdrLoad,u			; subtract start address, calculate exec offset from start
		stx		HdrLoad,u			; save new load address in header
		addd	HdrLoad,u			; calculate new exec address
		std		BasExecAddr			; set exec address

CmdMLoadBinDef		
		lbsr	MMC_LoadFile		; load the file
		lbsr	MMC_CloseFile		; close the file

CmdMLoadExit
		leas	FileHeadLen,s		; cleanup stack frame
		puls	a,u					; recover auto flag
		cmpa	#$41				; auto-run ?
		bne		CmdMLoadBinExit		; nope : return

		jmp		[<BasExecAddr]		; Run the loaded code

CmdMLoadBinExit		
		clra
		rts							; restore / return

CmdMLoadBas
		bsr		LoadBasic			; load basic prog into memory
		
		ldx		<BasStartProg		; get base of loaded prog
		jsr		BasSetProgPtrX		; setup
		
		ldx		<AddrFWareRamTop	; get ramtop
		stx		<BasVarStrTop		; set top of string area
		ldx		<BasVarSimpleAddr	; get simple vars addr
		stx		<BasVarArrayAddr	; set array vars to be the same
		stx		<BasVarEnd			; set end of basic vars
		
		jsr		CmdRestore			; do a restore

		leas	FileHeadLen,s		; cleanup stack frame
		puls	a,u					; recover auto flag (a)

		jsr		BasResetStack		; reset the stack a preserved
		
		cmpa	#$41				; Auto run ?
		bne		CmdMLoadBasExit		; nope : return to command mode
		
		clra	
		jmp		BasRun				; run it
		
CmdMLoadBasExit
		CLRA
        JMP     >BasCmdMode

CmdMLoadError
		lbsr	MMC_CloseFile		; close the file

		leas	FileHeadLen,s		; cleanup stack frame
		puls	a,u					; recover auto flag (a)

		clra						; flag error
		coma
		rts
		
LoadBasic
		ldd		HdrLen,u			; get file length
		addd	<BasStartProg		; add start of basic text
		std		<BasVarEnd			; set end of storage
		
		ldb		#$40				; check to see there's enough memory
		jsr		BasChkB2Free
	
		ldx		<BasStartProg		; get load address
		stx		HdrLoad,u			; save in header
	
		lbsr	MMC_LoadFile		; load the file
	
		leax	2,x					; setup vars
		stx		<BasVarSimpleAddr
		lbsr	MMC_CloseFile		; close the file
		rts
		
;
; TestAutoBoot, test if auto-booting is enabled, and try to boot if it is.
;

TestAutoBoot
		lda		<CFGByte				; Get config byte
		
		anda	#CFG_ENABLE_AUTOBOOT ; auto-boot enabled ?
		beq		NoAutoBoot			; nope return
		
		ldb		#AutoFileNameLen	; Length of filename
		leax	AutoFileName,pcr	; Point to filename
		lbsr	MMC_SendName		; send it to MMC
		
		lda		#$41				; Auto boot 'flag'
		pshs	u,a					; stack them
		
		lbra	CmdMLoadAutoBoot	; Attempt to open / load
		
NoAutoBoot
		clra	
		rts

AutoFileName		FCC		'AUTOEXEC'
AutoFileNameLen		EQU		*-AutoFileName


TestName	FCC	"CH"
TestNameLen	EQU	*-TestName

MSaveTest
		leax	TestName,PCR		; send name
		ldb		#TestNameLen
		lbsr	MMC_SendName

		leas	-FileHeadLen,s		; temp stack frame
		leau	,s

		ldx		#$600
		stx		HdrLoad,u
		
		ldx		#$7501
		tfr		x,d
		
		subd	HdrLoad,u		; Convert to length
		addd	#1				; fixup 
		std		HdrLen,u		; Save length
		
		ldx		#$1
		stx		HdrExec,u
		
		lda		#FTypeBin		; Set as binary
		sta		HdrType,u		
		
		lda		#$55				; Save ID bytes
		sta		HdrID55,u
		coma
		sta		HdrIDAA,u
		
		lda		#CMD_FILE_OPEN_WRITE ; Open file
		lbsr	MMC_SendCmd			; Open file
		lbsr	CheckError			; Bomb out on error
		
		lbsr	MMC_SaveFile		; Save the file
		
		lbsr	MMC_CloseFile		; close the file

		leas	FileHeadLen,s		; clean up
		clra
		rts
		