;
; Retokenize basic between Dragon and CoCo
;
; Majority of code borrowed from MultiCartBoot.
;

CmdRetok:
		cmpa	#'D					; RetokD CoCo->Dragon ?
		beq		CmdRetokToDragon	; yep, do it
		cmpa	#'C					; RetokC Dragon->CoCo ?
		beq		CmdRetokToCoCo		; yep, do it
		clra						; no error
		rts

; retokenize CoCo to Dragon.
CmdRetokToDragon:
		jsr		<BasChrGet			; skip the D
		leax	TokDragonToCoCo,pcr	; point at token tables
		bra		DoRetok				; Retokenize

; retokenize Dragon to CoCo.
CmdRetokToCoCo:
		jsr		<BasChrGet			; skip the C
		leax	TokCoCoToDragon,pcr	; point at token tables
		bra		DoRetok				; Retokenize

DoRetok
		ldu		,x					; Get token lookup ptr
		ldy		BasStartProg		; Get start of basic 
	
DoRetokNewLine
		leay	4,y					; skip link & lineno
RetokLoop
		lda		,y					; get a byte
	
		tsta						; Test a and set flags
		beq		DoRetokEol			; end of line, check for end of code
	
		bmi		LookupToken			; Token, yes process it

RetokNextByte
		leay	1,y					; increment pointer
		bra		RetokLoop			; do next
	
LookupToken
		cmpa	#$FF				; Check extended token flag
		beq		RetokNextByte		; yes : do next
	
		ldb		-1,y				; Get previous byte
		cmpb	#$FF				; Extended token flag ?
		beq		DoExtended			; Yes do it
		ldu		,x					; Get normal function pointer
		bra		DoTranslate			; skip

DoExtended
		ldu		2,x					; Point at extended token table

DoTranslate
		anda	#$7f				; Make it zero based
		ldb		a,u					; get new token
		stb		,y					; replace it
		bra		RetokNextByte		; do next
	
DoRetokEol
		leay	1,y					; increment pointer
		tst		,y					; check for a second zero, marking end of program
		bne		DoRetokNewLine		; no : do next line
		clra						; no error
		rts


;***********************************************
; Token translation tables for basic programs. *
;***********************************************

TokDragonToCoCo
	FDB	DragonCoCoCmd
	FDB	DragonCoCoFunc
	
TokCoCoToDragon
	FDB	CoCoDragonCmd
	FDB	CoCoDragonFunc

;*****************
; Dragon to CoCo *
;*****************

DragonCoCoCmd
	FCB	$80
	FCB	$81
	FCB	$82
	FCB	$83
	FCB	$84
	FCB	$85
	FCB	$86
	FCB	$87
	FCB	$88
	FCB	$89
	FCB	$8a
	FCB	$8b
	FCB	$8c
	FCB	$8d
	FCB	$ba
	FCB	$8e
	FCB	$8f
	FCB	$90
	FCB	$91
	FCB	$92
	FCB	$93
	FCB	$94
	FCB	$95
	FCB	$96
	FCB	$b9
	FCB	$97
	FCB	$98
	FCB	$99
	FCB	$9a
	FCB	$9b
	FCB	$9c
	FCB	$9d
	FCB	$9e
	FCB	$9f
	FCB	$a0
	FCB	$a1
	FCB	$a2
	FCB	$a3
	FCB	$b5
	FCB	$b6
	FCB	$b7
	FCB	$b8
	FCB	$bb
	FCB	$bc
	FCB	$bd
	FCB	$be
	FCB	$bf
	FCB	$c0
	FCB	$c1
	FCB	$c2
	FCB	$c3
	FCB	$c4
	FCB	$c5
	FCB	$c6
	FCB	$c7
	FCB	$c8
	FCB	$c9
	FCB	$ca
	FCB	$cb
	FCB	$a4
	FCB	$a5
	FCB	$a6
	FCB	$cc
	FCB	$a7
	FCB	$a8
	FCB	$a9
	FCB	$aa
	FCB	$ab
	FCB	$ac
	FCB	$ad
	FCB	$ae
	FCB	$af
	FCB	$b0
	FCB	$b1
	FCB	$b2
	FCB	$b3
	FCB	$b4
	FCB	$cd

DragonCoCoFunc
	FCB	$80
	FCB	$81
	FCB	$82
	FCB	$9a
	FCB	$84
	FCB	$9b
	FCB	$99
	FCB	$97
	FCB	$85
	FCB	$95
	FCB	$96
	FCB	$94
	FCB	$86
	FCB	$87
	FCB	$88
	FCB	$89
	FCB	$8a
	FCB	$8b
	FCB	$8c
	FCB	$8d
	FCB	$98
	FCB	$9c
	FCB	$8e
	FCB	$8f
	FCB	$90
	FCB	$91
	FCB	$92
	FCB	$93
	FCB	$9d
	FCB	$9e
	FCB	$9f
	FCB	$a0
	FCB	$a1
	FCB	$83

;*****************
; CoCo to Dragon *
;*****************

CoCoDragonCmd
	FCB	$80
	FCB	$81
	FCB	$82
	FCB	$83
	FCB	$84
	FCB	$85
	FCB	$86
	FCB	$87
	FCB	$88
	FCB	$89
	FCB	$8a
	FCB	$8b
	FCB	$8c
	FCB	$8d
	FCB	$8f
	FCB	$90
	FCB	$91
	FCB	$92
	FCB	$93
	FCB	$94
	FCB	$95
	FCB	$96
	FCB	$97
	FCB	$99
	FCB	$9a
	FCB	$9b
	FCB	$9c
	FCB	$9d
	FCB	$9e
	FCB	$9f
	FCB	$a0
	FCB	$a1
	FCB	$a2
	FCB	$a3
	FCB	$a4
	FCB	$a5
	FCB	$bb
	FCB	$bc
	FCB	$bd
	FCB	$bf
	FCB	$c0
	FCB	$c1
	FCB	$c2
	FCB	$c3
	FCB	$c4
	FCB	$c5
	FCB	$c6
	FCB	$c7
	FCB	$c8
	FCB	$c9
	FCB	$ca
	FCB	$cb
	FCB	$cc
	FCB	$a6
	FCB	$a7
	FCB	$a8
	FCB	$a9
	FCB	$98
	FCB	$8e
	FCB	$aa
	FCB	$ab
	FCB	$ac
	FCB	$ad
	FCB	$ae
	FCB	$af
	FCB	$b0
	FCB	$b1
	FCB	$b2
	FCB	$b3
	FCB	$b4
	FCB	$b5
	FCB	$b6
	FCB	$b7
	FCB	$b8
	FCB	$b9
	FCB	$ba
	FCB	$be
	FCB	$cd

CoCoDragonFunc
	FCB	$80
	FCB	$81
	FCB	$82
	FCB	$a1
	FCB	$84
	FCB	$88
	FCB	$8c
	FCB	$8d
	FCB	$8e
	FCB	$8f
	FCB	$90
	FCB	$91
	FCB	$92
	FCB	$93
	FCB	$96
	FCB	$97
	FCB	$98
	FCB	$99
	FCB	$9a
	FCB	$9b
	FCB	$8b
	FCB	$89
	FCB	$8a
	FCB	$87
	FCB	$94
	FCB	$86
	FCB	$83
	FCB	$85
	FCB	$95
	FCB	$9c
	FCB	$9d
	FCB	$9e
	FCB	$9f
	FCB	$a0

